//+------------------------------------------------------------------+
//|                                      PriceChannel_StopSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "PriceChannel_Stop.ex5"
#include <Expert\ExpertSignal.mqh>
//--- wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,   PriceChannel_Stop         |
//| Type=SignalAdvanced                                              |
//| Name=PriceChannel_Stop                                           |
//| Class=CPriceChannel_StopSignal                                   |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=ChannelPeriod,uint,5,                      |
//| Parameter=Risk,double,0.1,                           |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
//--- wizard description end
//+------------------------------------------------------------------+
//|  CPriceChannel_StopSignal.                                  |
//| :          |
//| PriceChannel_Stop http://www.mql5.com/ru/code/417/.              |
//|     CExpertSignal.                    |
//+------------------------------------------------------------------+
class CPriceChannel_StopSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      PriceChannel_Stop

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     

   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   ASCtrend   
   uint              m_ChannelPeriod;    //  
   double            m_Risk;             //  

   uint              m_SignalBar;        //      

public:
                     CPriceChannel_StopSignal();

   //---    
   void               BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void               SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void               BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void               SellPosClose(bool value)                { m_SellPosClose=value;     }

   //---   
   void               Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void               ChannelPeriod(uint value)               { m_ChannelPeriod=value;    }
   void               Risk(double value)                      { m_Risk=value;             }
   //---
   void               SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitPriceChannel_Stop(CIndicators *indicators);  // ,   PriceChannel_Stop
  };
//+------------------------------------------------------------------+
//|  CPriceChannel_StopSignal.                            |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CPriceChannel_StopSignal::CPriceChannel_StopSignal()
  {
//---    
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;

//---   
   m_Ind_Timeframe=PERIOD_H4;
   m_ChannelPeriod=5;
   m_Risk=0.1;
//---  
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true -   ,  false.           |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CPriceChannel_StopSignal::ValidationSettings()
  {
//---  
   if(m_ChannelPeriod<=0)
     {
      printf(__FUNCTION__+":       ");
      return(false);
     }

   if(m_Risk<=0)
     {
      printf(__FUNCTION__+":       ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true -    ,  false.       |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CPriceChannel_StopSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitPriceChannel_Stop(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   PriceChannel_Stop.                      |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true -    ,  false.       |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CPriceChannel_StopSignal::InitPriceChannel_Stop(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[3];

   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="PriceChannel_Stop.ex5";

   parameters[1].type=TYPE_UINT;
   parameters[1].integer_value=m_ChannelPeriod;

   parameters[2].type=TYPE_DOUBLE;
   parameters[2].double_value=m_Risk;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,3,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---  
   if(!m_indicator.NumBuffers(6)) return(false);

//---     PriceChannel_Stop
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CPriceChannel_StopSignal::LongCondition()
  {
//---       3  PriceChannel_Stop
   double Signal=m_indicator.GetData(3,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_BuyPosOpen)
        {
         if(m_SellPosClose) return(REVERSE_SHORT);
         else return(OPEN_LONG);
        }
      else
        {
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }

//---      
   if(!m_SellPosClose) return(NO_SIGNAL);

//---      4  PriceChannel_Stop
   Signal=m_indicator.GetData(4,m_SignalBar);
   if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_SHORT);

//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CPriceChannel_StopSignal::ShortCondition()
  {
//---       0  PriceChannel_Stop
   double Signal=m_indicator.GetData(0,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_SellPosOpen)
        {
         if(m_BuyPosClose) return(REVERSE_LONG);
         else return(OPEN_SHORT);
        }
      else
        {
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }

//---      
   if(!m_BuyPosClose) return(NO_SIGNAL);

//---      1  PriceChannel_Stop
   Signal=m_indicator.GetData(1,m_SignalBar);
   if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_LONG);

//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
